/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Software Author: Tony Whyman mailto:tony.whyman@mwasoftware.co.uk
   http://www.mwasoftware.co.uk


*/

#include <stdlib.h>
#include <windows.h>
#include <msiquery.h>  //Supplied as part of the MS Platform SDK
#include <ver.h>
#include <dir.h>
#include <stdio.h>

//Error numbers used as keys into the Merge Module's Error Table

#define NOSERVICEDBPATH 27014
#define INTERNALERROR1 27020
#define MOVEFAILED 27021
#define COPYFAILED 27022

//Property Names
#define IBRUNNING "IBRUNNING.7B827C0C_A4D6_4FA8_808D_CEE88C577183"
#define FBRUNNING "FBRUNNING.7B827C0C_A4D6_4FA8_808D_CEE88C577183"
#define SRCFILE "SRCFILE.3CCBBB43_06CD_4BB0_83FE_72DA0FA2AE08"
#define DESTFILE "DESTFILE.3CCBBB43_06CD_4BB0_83FE_72DA0FA2AE08"

//The following is copied from iscguard/iscguard.rh in the InterBase source
#define IDM_SHUTDOWN 5

#define SERVER_CLASS_NAME "FB_Server"
#define SERVER_APP_NAME "Firebird Server"
#define GUARDIAN_APP_NAME		"Firebird Guardian"
#define GUARDIAN_CLASS_NAME		"FB_Guard"

//-----------------------------------------------------------------------------
//Display a error message with no parameters

void Error(MSIHANDLE hInstall, int Message){
MSIHANDLE hRec;

  hRec = MsiCreateRecord(2);
  __try {
    MsiRecordSetInteger(hRec,1,Message);
    MsiProcessMessage(hInstall,INSTALLMESSAGE_ERROR,hRec);
  }
  __finally {
    MsiCloseHandle(hRec);
  }
}

//-----------------------------------------------------------------------------
//Display a Fatal Error Message with no parameters

void FatalError(MSIHANDLE hInstall, int Message){
MSIHANDLE hRec;

  hRec = MsiCreateRecord(2);
  __try {
    MsiRecordSetInteger(hRec,1,Message);
    MsiProcessMessage(hInstall,INSTALLMESSAGE_FATALEXIT,hRec);
  }
  __finally {
    MsiCloseHandle(hRec);
  }
}

//-----------------------------------------------------------------------------
//Display an error message with the "Filename" as the parameter

void Error1(MSIHANDLE hInstall, int Message, LPTSTR Filename){
MSIHANDLE hRec;

  hRec = MsiCreateRecord(3);
  __try {
    MsiRecordSetInteger(hRec,1,Message);
    MsiRecordSetString(hRec,2,(LPCTSTR) Filename);
    MsiProcessMessage(hInstall,INSTALLMESSAGE_ERROR,hRec);
  }
  __finally {
    MsiCloseHandle(hRec);
  }
}

//-----------------------------------------------------------------------------
//Display a Warning Message with no parameters

void Warning(MSIHANDLE hInstall, int Message){
MSIHANDLE hRec;

  hRec = MsiCreateRecord(2);
  __try {
    MsiRecordSetInteger(hRec,1,Message);
    MsiProcessMessage(hInstall,INSTALLMESSAGE_WARNING,hRec);
  }
  __finally {
    MsiCloseHandle(hRec);
  }
}

//-----------------------------------------------------------------------------
//If the IB Guardian or Server is running then display an error message and
//force termination of the installation

UINT __stdcall IsIBRunning(MSIHANDLE hInstall)
{
  if (FindWindow("IB_Server", "InterBase Server")
    || FindWindow("IB_Guard", "InterBase Guardian")) {

   MsiSetProperty(hInstall,IBRUNNING,"Yes");
  }
  return ERROR_SUCCESS;

}
//-----------------------------------------------------------------------------
//If the Firebird Guardian or Server is running then display an error message and
//force termination of the installation

UINT __stdcall IsFBRunning(MSIHANDLE hInstall)
{
  if (FindWindow(SERVER_CLASS_NAME, SERVER_APP_NAME)
    || FindWindow(GUARDIAN_CLASS_NAME, GUARDIAN_APP_NAME)) {

   MsiSetProperty(hInstall,FBRUNNING,"Yes");
  }
  return ERROR_SUCCESS;

}
//-----------------------------------------------------------------------------
//Locate and stop the InterBase or Firebird 1 Server

UINT __stdcall IBStop(MSIHANDLE hInstall)
{
  HWND hTmpWnd;
  hTmpWnd = FindWindow("IB_Guard", "InterBase Guardian");
  if (hTmpWnd == NULL)
    hTmpWnd = FindWindow("IB_Server", "InterBase Server");
  if (hTmpWnd != NULL)
    PostMessage(hTmpWnd, WM_COMMAND, (WPARAM) IDM_SHUTDOWN, 0);
  return ERROR_SUCCESS;

}
//-----------------------------------------------------------------------------
//Locate and stop the Firebird 1.5 Server

UINT __stdcall FBStop(MSIHANDLE hInstall)
{
  HWND hTmpWnd;
  hTmpWnd = FindWindow(GUARDIAN_CLASS_NAME, GUARDIAN_APP_NAME);
  if (hTmpWnd == NULL)
    hTmpWnd = FindWindow(SERVER_CLASS_NAME, SERVER_APP_NAME);
  if (hTmpWnd != NULL)
    PostMessage(hTmpWnd, WM_COMMAND, (WPARAM) IDM_SHUTDOWN, 0);
  return ERROR_SUCCESS;

}
//-----------------------------------------------------------------------------
// Returns 1 if FileName is a valid path to an existing file and 0 otherwise

int FileExists(char *FileName) {

WIN32_FIND_DATA Data;
HANDLE SearchHandle;

  if ((SearchHandle = FindFirstFile(FileName,&Data))
    == INVALID_HANDLE_VALUE)
    return 0;

  FindClose(SearchHandle);
  return 1;
}
//-----------------------------------------------------------------------------
// Work out where the Services Database is.

int GetServiceDBPath(char *ServiceDBPath) {

char WinDir[MAXPATH];

  GetWindowsDirectory(&WinDir,MAXPATH);

  //Try Win9x first

  if (FileExists(strcat(strcpy(ServiceDBPath,&WinDir),"\\Services")))
    return 1;
  // try WinNT

  if (FileExists(strcat(strcpy(ServiceDBPath,&WinDir),"\\System32\\drivers\\etc\\Services")))
    return 1;

  return 0;

}

//-----------------------------------------------------------------------------
//Check the Services file for th gds_db entry and add one if not found.

void AddGDSEntry(char *ServiceDBPath) {

FILE *Services;
char line[128];

  if ((Services = fopen(ServiceDBPath,"rt")) != NULL) {
    __try {
      while (fgets(&line,128,Services)) {
        if (strncmp(&line,"gds_db",6) == 0) return;
      }
    }
    __finally {
      fclose(Services);
    }
  }

  if ((Services = fopen(ServiceDBPath,"at")) != NULL) {
    __try {
      fputs("\ngds_db\t3050/tcp\n",Services);
    }
    __finally {
      fclose(Services);
    }
  }
}

//-----------------------------------------------------------------------------
//Determines where the Services file is and then calls AddGDSEntry.

UINT __stdcall AddService(MSIHANDLE hInstall) {

char ServiceDBPath[MAXPATH];

  if (GetServiceDBPath(&ServiceDBPath)) {
    AddGDSEntry(&ServiceDBPath);
  }
  else
   Error(hInstall,NOSERVICEDBPATH);
  return ERROR_SUCCESS;
}

//-----------------------------------------------------------------------------
//Moves file

UINT __stdcall FBMoveFile(MSIHANDLE hInstall) {

char sourcefile[MAXPATH];
char destfile[MAXPATH];
LPVOID lpMsgBuf;
DWORD sourceSize = sizeof(sourcefile);
DWORD destSize = sizeof(destfile);


if (MsiGetProperty(hInstall,SRCFILE,&sourcefile,&sourceSize) == ERROR_SUCCESS &&
   MsiGetProperty(hInstall,DESTFILE,&destfile,&destSize) == ERROR_SUCCESS) {
     if (MoveFileEx(&sourcefile,&destfile,MOVEFILE_REPLACE_EXISTING) == 0) {
       FormatMessage( 
    		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
    		NULL,
    		GetLastError(),
    		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
    	  (LPTSTR) &lpMsgBuf,
    		0,
    		NULL
			);


       Error1(hInstall,MOVEFAILED,(LPTSTR)lpMsgBuf);
     }
}
else
  Error(hInstall,INTERNALERROR1);
return ERROR_SUCCESS;
}

//-----------------------------------------------------------------------------
//Copies file

UINT __stdcall FBCopyFile(MSIHANDLE hInstall) {

char sourcefile[MAXPATH];
char destfile[MAXPATH];
LPVOID lpMsgBuf;
DWORD sourceSize = sizeof(sourcefile);
DWORD destSize = sizeof(destfile);


if (MsiGetProperty(hInstall,SRCFILE,&sourcefile,&sourceSize) == ERROR_SUCCESS &&
   MsiGetProperty(hInstall,DESTFILE,&destfile,&destSize) == ERROR_SUCCESS) {
     if (CopyFile(&sourcefile,&destfile,FALSE) == 0) {
       FormatMessage( 
    		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
    		NULL,
    		GetLastError(),
    		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
    	  (LPTSTR) &lpMsgBuf,
    		0,
    		NULL
			);


       Error1(hInstall,COPYFAILED,(LPTSTR)lpMsgBuf);
     }
}
else
  Error(hInstall,INTERNALERROR1);
return ERROR_SUCCESS;
}


